#include <assert.h>
#include "defs/defs.h"	//VECTOR_DELETE_ALL
#include "shader/program.h"

void ShaderProgram::create() {
	handle = glCreateProgramObjectARB();
}

void ShaderProgram::destroy() {
	VECTOR_DELETE_ALL(internalShaderList);
	allShaderList.resize(0);
	super::destroy();
}

void ShaderProgram::attach(GLhandleARB extHandle) {
	assert(handle);
	glAttachObjectARB(handle, extHandle);
	allShaderList.push_back(extHandle);
}

void ShaderProgram::attach(const Shader &shader) {
	assert(handle);
	glAttachObjectARB(handle, shader.getHandle());
	allShaderList.push_back(shader.getHandle());
}

bool ShaderProgram::link() {
	glLinkProgramARB(handle);
	GLint linked = 0;
	glGetObjectParameterivARB(handle, GL_OBJECT_LINK_STATUS_ARB, &linked);
	return !!linked;
}

void ShaderProgram::use() {
	glUseProgramObjectARB(handle);
}

void ShaderProgram::useNone() {
	glUseProgramObjectARB(0);	 //is this as unsafe as glBindTexture(***, 0) is?
}

int ShaderProgram::getUniformLocation(const char *name) {
	return glGetUniformLocationARB(handle, name);
}

void ShaderProgram::getInfoLog(int logsize, int *res, char *log) {
	glGetInfoLogARB(handle, logsize, (GLsizei *)res, log);	//safe conversion from GLsizei* to int* ?
}

void ShaderProgram::setUniform1i(const char *name, int value) {
	use();
	glUniform1iARB(getUniformLocation(name), value);
}

void ShaderProgram::setUniform1f(const char *name, float value) {
	use();
	glUniform1fARB(getUniformLocation(name), value);
}

bool ShaderProgram::attachShader(const char *filename, int shaderType, const char *prefixString) {
	Shader *shader = new Shader();
	if (!shader) return false;
	if (!shader->createFromSource(filename, shaderType, prefixString)) {
		delete shader;
		return false;
	}
	internalShaderList.push_back(shader);
	attach(shader->getHandle());
	return true;
}

bool ShaderProgram::attachVertexShader(const char *filename, const char *prefixString)	{
	return attachShader(filename, GL_VERTEX_SHADER_ARB, prefixString);
}

bool ShaderProgram::attachFragmentShader(const char *filename, const char *prefixString) {
	return attachShader(filename, GL_FRAGMENT_SHADER_ARB, prefixString);
}

void ShaderProgram::printLog() {
	for (int i = 0; i < (int)allShaderList.size(); i++) {
		printHandleLog(allShaderList[i]);
	}
	printHandleLog(handle);
}
